@@ -1,5 +1,3 @@
-import Path from './graphic/Path';
-import ZRImage from './graphic/Image';
 import { GradientObject } from './graphic/Gradient';
 import { PatternObject } from './graphic/Pattern';
 import { Dictionary } from './core/types';
@@ -13,7 +11,11 @@ export interface PainterBase {
 
     type: string
 
-    root: HTMLElement
+    // root will be undefined if ssr is true
+    root?: HTMLElement
+
+    // If ssr only
+    ssrOnly?: boolean
 
     // constructor(dom: HTMLElement, storage: Storage, opts: PainterOption, id: number): void
 
@@ -21,6 +23,9 @@ export interface PainterBase {
     refresh(): void
     clear(): void
 
+    // must be given if ssr is true.
+    renderToString?(): string;
+
     getType: () => string
 
     getWidth(): number
@@ -31,7 +36,6 @@ export interface PainterBase {
     getViewportRootOffset: () => {offsetLeft: number, offsetTop: number}
 
     refreshHover(): void
-    pathToImage(e: Path, dpr: number): ZRImage
 
     configLayer(zlevel: number, config: Dictionary<any>): void
     setBackgroundColor(backgroundColor: string | GradientObject | PatternObject): void
